@@ -1,7 +1,10 @@
 from __future__ import print_function
 
 name = "csinsc"
-version = "1.0.0"
+version = "1.0.3.3"
+
+# for complete the blanks exercises
+__________ = None
 
 '''
 goto functionality
@@ -16,10 +19,12 @@ checking_argv0 = True
 
 import sys, token, tokenize
 
+
 class MissingLabelError(Exception):
     """'goto' without matching 'label'."""
     pass
 
+
 # Source filenames -> line numbers of plain gotos -> target label names.
 _plainGotoCache = {}
 # Source filenames -> line numbers of labels -> label names.
@@ -27,6 +32,7 @@ _labelCache = {}
 # Source filenames -> label names -> line numbers of those labels.
 _labelNameCache = {}
 
+
 def _addToCaches(moduleFilename):
     """Finds the labels and gotos in a module and adds them to the caches."""
 
@@ -63,11 +69,11 @@ def _trace(frame, event, arg):
         # If this is the first time we've seen this source file, cache it.
         filename = frame.f_code.co_filename
         if checking_argv0:
-          if filename != sys.argv[0]:
-              return
+            if filename != sys.argv[0]:
+                return
         else:
-          if filename[0] == "<":
-              return
+            if filename[0] == "<":
+                return
 
         if filename not in _plainGotoCache:
             _addToCaches(filename)
@@ -88,6 +94,7 @@ def _trace(frame, event, arg):
 
     return _trace
 
+
 # Install the trace function, including all preceding frames.
 sys.settrace(_trace)
 frame = sys._getframe().f_back
@@ -95,6 +102,7 @@ while frame:
     frame.f_trace = _trace
     frame = frame.f_back
 
+
 # Define the so-called keywords for importing: 'goto', 'label' and 'comefrom'.
 class _Label:
     """Allows arbitrary x.y attribute lookups."""
@@ -102,9 +110,11 @@ class _Label:
     def __getattr__(self, name):
         return None
 
+
 goto = None
 label = _Label()
 
+
 ######################### color constants #########################
 #             abstracted for both termcolour and curses           #
 ###################################################################
@@ -119,6 +129,7 @@ class Colour:
     white = '\033[37m'
     reset = '\033[0m'
 
+
 class Highlight:
     grey = '\033[40m'
     red = '\033[41m'
@@ -129,6 +140,7 @@ class Highlight:
     cyan = '\033[46m'
     white = '\033[47m'
 
+
 class Style:
     bold = '\033[1m'
     dark = '\033[2m'
@@ -137,12 +149,10 @@ class Style:
     reverse = '\033[7m'
     concealed = '\033[8m'
 
+
 ############### color lookup for curses ###############
 curses_lookup = {}
 
-
-
-
 # coding: utf-8
 # Copyright (c) 2008-2011 Volvox Development Team
 #
@@ -168,62 +178,57 @@ curses_lookup = {}
 
 """ANSII Color formatting for output in terminal."""
 
-
 import os
 
-
-__ALL__ = [ 'colored', 'cprint' ]
+__ALL__ = ['colored', 'cprint']
 
 VERSION = (1, 1, 0)
 
 ATTRIBUTES = dict(
-        list(zip([
-            'bold',
-            'dark',
-            '',
-            'underline',
-            'blink',
-            '',
-            'reverse',
-            'concealed'
-            ],
-            list(range(1, 9))
-            ))
-        )
+    list(zip([
+        'bold',
+        'dark',
+        '',
+        'underline',
+        'blink',
+        '',
+        'reverse',
+        'concealed'
+    ],
+        list(range(1, 9))
+    ))
+)
 del ATTRIBUTES['']
 
-
 HIGHLIGHTS = dict(
-        list(zip([
-            'on_grey',
-            'on_red',
-            'on_green',
-            'on_yellow',
-            'on_blue',
-            'on_magenta',
-            'on_cyan',
-            'on_white'
-            ],
-            list(range(40, 48))
-            ))
-        )
-
+    list(zip([
+        'on_grey',
+        'on_red',
+        'on_green',
+        'on_yellow',
+        'on_blue',
+        'on_magenta',
+        'on_cyan',
+        'on_white'
+    ],
+        list(range(40, 48))
+    ))
+)
 
 COLORS = dict(
-        list(zip([
-            'grey',
-            'red',
-            'green',
-            'yellow',
-            'blue',
-            'magenta',
-            'cyan',
-            'white',
-            ],
-            list(range(30, 38))
-            ))
-        )
-
+    list(zip([
+        'grey',
+        'red',
+        'green',
+        'yellow',
+        'blue',
+        'magenta',
+        'cyan',
+        'white',
+    ],
+        list(range(30, 38))
+    ))
+)
 
 RESET = '\033[0m'
 
@@ -301,235 +306,250 @@ if __name__ == '__main__':
     cprint('Reversed blue color', 'blue', attrs=['reverse'])
     cprint('Concealed Magenta color', 'magenta', attrs=['concealed'])
     cprint('Bold underline reverse cyan color', 'cyan',
-            attrs=['bold', 'underline', 'reverse'])
+           attrs=['bold', 'underline', 'reverse'])
     cprint('Dark blink concealed white color', 'white',
-            attrs=['dark', 'blink', 'concealed'])
+           attrs=['dark', 'blink', 'concealed'])
     print(('-' * 78))
 
     print('Test mixing:')
     cprint('Underline red on grey color', 'red', 'on_grey',
-            ['underline'])
+           ['underline'])
     cprint('Reversed green on red color', 'green', 'on_red', ['reverse'])
 
 from time import *
 
+
 class SimpleScreen:
-  def __init__(self):
-    self.width = 10
-    self.height = 5
-    self.num_lines = 100
-    
-    self.screen = [[] * self.width] * self.height
-    self.clear()
+    def __init__(self):
+        self.width = 10
+        self.height = 5
+        self.num_lines = 100
+
+        self.screen = [[] * self.width] * self.height
+        self.clear()
+
+        self.fps = 60
+
+    def clear(self):
+        for row in range(self.height):
+            self.screen[row] = [' '] * self.width
+
+    def print_at(self, text, x, y):
+        self.screen[y][x] = text
+
+    def show(self):
+
+        for _ in range(self.num_lines):
+            print()
+        print(flush=True)
+
+        for row in self.screen:
+            print("".join(row))
+        print(flush=True)
 
-    self.fps = 60
-    
-  def clear(self):
-    for row in range(self.height):
-      self.screen[row] = [' '] * self.width
 
-  def print_at(self, text, x, y):
-    self.screen[y][x] = text
-    
-  def show(self):
-    
-    for _ in range(self.num_lines):
-      print()
-    print(flush = True)
-    
-    for row in self.screen:
-      print("".join(row))
-    print(flush = True)
-    
 ####################################################################
 # Simple curses wrapper so console based text games can be easily  #
 # written, tweaked, and analysed.                                  #
 ####################################################################
-def simple_clear(lines = 40):
+def simple_clear(lines=40):
     for _ in range(lines):
         print()
 
 
 from curses import *
 
+
 class Screen(object):
-  def __init__(self, width = 40, height = 25, fps = 60, auto_setup = True, refresh_on_clear = True):
-    self.width = width
-    self.height = height
+    def __init__(self, width=40, height=25, colour=False, fps=60, auto_setup=True, refresh_on_clear=True):
+        self.width = width
+        self.height = height
+        self.colourMode = colour
 
-    self.screen = [[] * self.width] * self.height
+        self.screen = [[] * self.width] * self.height
 
-    self.colour_screen = [[] * self.width] * self.height
+        self.colour_screen = [[] * self.width] * self.height
 
+        self.keys = {}
+        for key in range(512):
+            self.keys[key] = False
 
-    self.keys = {}
-    for key in range(512):
-      self.keys[key] = False
+        self.win = None
 
-    self.win = None
+        self.fps = fps
 
-    self.fps = fps
+        self.x = 0
+        self.y = 0
 
-    self.x = 0
-    self.y = 0
+        self.refresh_on_clear = refresh_on_clear
 
-    self.refresh_on_clear = refresh_on_clear
+        # 0:black, 1:red, 2:green, 3:yellow, 4:blue, 5:magenta, 6:cyan, and 7:white
+        self.colours = {}
+        self.colours[Colour.grey] = 0
+        self.colours[Colour.red] = 1
+        self.colours[Colour.green] = 2
+        self.colours[Colour.yellow] = 3
+        self.colours[Colour.blue] = 4
+        self.colours[Colour.magenta] = 5
+        self.colours[Colour.cyan] = 6
+        self.colours[Colour.white] = 7
 
-    #0:black, 1:red, 2:green, 3:yellow, 4:blue, 5:magenta, 6:cyan, and 7:white
-    self.colours = {}
-    self.colours[Colour.grey] = 0
-    self.colours[Colour.red] = 1
-    self.colours[Colour.green] = 2
-    self.colours[Colour.yellow] = 3
-    self.colours[Colour.blue] = 4
-    self.colours[Colour.magenta] = 5
-    self.colours[Colour.cyan] = 6
-    self.colours[Colour.white] = 7
+        self.highlights = {}
+        self.highlights[Highlight.grey] = 0
+        self.highlights[Highlight.red] = 1
+        self.highlights[Highlight.green] = 2
+        self.highlights[Highlight.yellow] = 3
+        self.highlights[Highlight.blue] = 4
+        self.highlights[Highlight.magenta] = 5
+        self.highlights[Highlight.cyan] = 6
+        self.highlights[Highlight.white] = 7
 
-    self.highlights = {}
-    self.highlights[Highlight.grey] = 0
-    self.highlights[Highlight.red] = 1
-    self.highlights[Highlight.green] = 2
-    self.highlights[Highlight.yellow] = 3
-    self.highlights[Highlight.blue] = 4
-    self.highlights[Highlight.magenta] = 5
-    self.highlights[Highlight.cyan] = 6
-    self.highlights[Highlight.white] = 7
+        if auto_setup:
+            self.setup()
 
-    if auto_setup:
+        self.clear()
+
+    def __del__(self):
+        if self.win is not None:
+            self.teardown()
+
+    def __enter__(self):
         self.setup()
+        self.clear()
+        return self
 
-    self.clear()
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.teardown()
 
-  def __del__(self):
-    if self.win is not None:
-      self.teardown()
+    def setup(self):
+        initscr()
+        noecho()
+        curs_set(0)
+        self.win = newwin(self.height + 1, self.width + 1, 0, 0)
+        self.win.keypad(1)
+        self.win.nodelay(1)
+        if self.colourMode:
+            start_color()
+            use_default_colors()
+            for color in range(8):
+                for bgcolor in range(8):
+                    init_pair(1 + (color * 8 + bgcolor), color, bgcolor)
 
-  def __enter__(self):
-    self.setup()
-    self.clear()
-    return self
+    def teardown(self):
+        self.win.keypad(0)
+        echo()
+        curs_set(1)
+        endwin()
 
-  def __exit__(self, exc_type, exc_val, exc_tb):
-    self.teardown()
+    def shutdown(self):
+        self.teardown()
 
-  def setup(self):
-    initscr()
-    noecho()
-    curs_set(0)
-    start_color()
-    self.win = newwin(self.height + 1, self.width + 1, 0, 0)
-    self.win.keypad(1)
-    self.win.nodelay(1)
+    def clear(self, clear_ch=" ", clear_col=Colour.white, clear_bg=Highlight.grey, reset_cursor=True):
+        # TODO: fix this so that black works
+        clear_col = 0
+        if self.refresh_on_clear:
+            self.refresh()
 
-    for color in range(8):
-        for bgcolor in range(8):
-            init_pair(1 + (color * 8 + bgcolor), color, bgcolor)
+        for row in range(self.height):
+            self.screen[row] = [clear_ch] * self.width
 
+        if self.colourMode:
+            for row in range(self.height):
+                self.colour_screen[row] = [clear_col] * self.width
 
-  def teardown(self):
-    self.win.keypad(0)
-    echo()
-    curs_set(1)
-    endwin()
+        if reset_cursor:
+            self.x = 0
+            self.y = 0
 
-  def shutdown(self):
-    self.teardown()
+    def getch(self):
+        return self.win.getch()
 
-  def clear(self, clear_ch = " ", clear_col = 0, reset_cursor = True):
-    if self.refresh_on_clear:
-      self.refresh()
-      
-    for row in range(self.height):
-      self.screen[row] = [clear_ch] * self.width
+    def print(self, text, newline=True):
+        x = int(self.x)
+        y = int(self.y)
 
-    for row in range(self.height):
-      self.colour_screen[row] = [clear_col] * self.width
+        for dx, ch in enumerate(text):
+            if x + dx > self.width:
+                # wrap to next line
+                x = x + dx - self.width
+                y = y + 1
+            if y > self.height:
+                # scroll everything up
+                for row in range(1, self.height):
+                    for col in range(self.width):
+                        self.screen[y - 1][col] = self.screen[y][col]
+            self.screen[y][x + dx] = ch
 
-    if reset_cursor:
-      self.x = 0
-      self.y = 0
+        if newline:
+            y += 1
 
+        self.x = x
+        self.y = y
 
- 
-  def getch(self):
-    return self.win.getch()
+        # if self.auto_refresh:
+        #  self.refresh()
 
-  def print(self, text, newline = True):
-    x = int(self.x)
-    y = int(self.y)
+    def border(self):
+        for x in range(1, self.width - 1):
+            self.screen[0][x] = '-'
+            self.screen[self.height - 1][x] = '-'
+        for y in range(1, self.height - 1):
+            self.screen[y][0] = '|'
+            self.screen[y][self.width - 1] = '|'
+        self.screen[0][0] = '.'
+        self.screen[0][self.width - 1] = '.'
+        self.screen[self.height - 1][0] = '\''
+        self.screen[self.height - 1][self.width - 1] = '\''
 
-    for dx, ch in enumerate(text):
-      if x + dx > self.width:
-        # wrap to next line
-        x = x + dx - self.width
-        y = y + 1
-      if y > self.height:
-        # scroll everything up
-        for row in range(1, self.height):
-          for col in range(self.width):
-            self.screen[y - 1][col] = self.screen[y][col]
-      self.screen[y][x + dx] = ch    
-      
-    if newline:
-      y += 1
+    def printAt(self, text, x, y, colour=None, bgcolour=None):
+        if color is None:
+            color = Colour.white
+        if bgcolor is None:
+            bgcolor = Highlight.grey
+        x = int(x)
+        y = int(y)
+        for dx, ch in enumerate(text):
+            self.screen[y][x + dx] = ch
 
-    self.x = x
-    self.y = y
+            if self.colourMode:
+                self.colour_screen[y][x + dx] = 1 + (self.colours[colour] * 8 \
+                                                     + self.highlights[bgcolour])
 
-    #if self.auto_refresh:
-    #  self.refresh()
+    def getCharAt(self, x, y):
+        return self.screen[y][x]
 
-  def border(self):
-    for x in range(1, self.width - 1):
-      self.screen[0][x] = '-'
-      self.screen[self.height - 1][x] = '-'
-    for y in range(1, self.height - 1):
-      self.screen[y][0] = '|'
-      self.screen[y][self.width - 1] = '|'
-    self.screen[0][0] = '.'
-    self.screen[0][self.width - 1] = '.'
-    self.screen[self.height - 1][0] = '\''
-    self.screen[self.height - 1][self.width - 1] = '\''    
+    def refresh(self):
+        if self.win is None:
+            self.setup()
 
-  def printAt(self, text, x, y, color = None, bgcolor = None):
-    if color is None:
-        color = Colour.white
-    if bgcolor is None:
-        bgcolor = Colour.grey
-    x = int(x)
-    y = int(y)
-    for dx, ch in enumerate(text):
-      self.screen[y][x + dx] = ch
-      self.colour_screen[y][x + dx] = 1 + (self.colours[color] * 8 \
-                                           + self.highlights[bgcolor])
+        for y, row in enumerate(self.screen):
+            if not self.colourMode:
+                text = "".join(row)
+                self.win.addstr(y, 0, text)
+            else:
+                for x, ch in enumerate(row):
+                    color = self.colour_screen[y][x]
+                    self.win.addstr(y, x, ch, color_pair(color))
 
+        for key in range(512):
+            self.keys[key] = False
 
-  def refresh(self):
-    if self.win is None:
-      self.setup()
-
-    for y, row in enumerate(self.screen):
-      line = str("".join(row))
-      self.win.addstr(y, 0, line)
-
-    for key in range(512):
-      self.keys[key] = False
-
-    self.win.timeout(int(1000.0 / self.fps))
-    ch = self.win.getch()
-    self.win.timeout(0)
-    while ch != -1:
-      self.keys[ch] = True
-      ch = self.win.getch()
-
-  def isKeyPressed(self, key):
-    return self.keys[ord(key)]
-
-  def setFPS(self, fps):
-    self.fps = fps
-    
-
+        self.win.timeout(int(1000.0 / self.fps))
+        ch = self.win.getch()
+        self.win.timeout(0)
+        while ch != -1:
+            self.keys[ch] = True
+            ch = self.win.getch()
 
+    def isKeyPressed(self, key):
+        return self.keys[ord(key)]
 
+    def getKeysPressed(self):
+        keys = []
+        for key in self.keys.keys():
+            if self.keys[key] == True:
+                keys.append(key)
+        return keys
 
+    def setFPS(self, fps):
+        self.fps = fps
